'use strict';

var _ = require('lodash');

var ConfigService = {
  getItems: function getItems(groups) {
    return _(groups).map(function (group) {
      return _(_.get(group, 'items', [])).map(function (item) {
        if (!_.isEmpty(item)) {
          if (item.type === 'select_many') {
            return _(_.get(item, 'items', [])).map(function (childItem) {
              if (!_.isEmpty(childItem)) {
                return childItem;
              }
            }).value();
          }
          return item;
        }
      }).value();
    }).flattenDeep().without(null).value();
  },

  getItem: function getItem(groups, itemName) {
    var item = null;
    _.some(ConfigService.getItems(groups), function (otherItem) {
      if (otherItem.name === itemName) {
        item = otherItem;
        return true;
      }
    });
    return item;
  },

  evaluateWhen: function evaluateWhen(groups, when) {
    var expanded = ConfigService.expandWhen(when);
    if (!expanded.key) {
      if (!expanded.value) {
        return true;
      }
      return ConfigService.parseBool(expanded.value) !== expanded.negate;
    }
    var theItem = ConfigService.getItem(groups, expanded.key);
    if (!theItem) {
      return true;
    }
    // recursively evaluate whens
    if (theItem.when && !ConfigService.evaluateWhen(groups, theItem.when)) {
      return false;
    }
    var value = _.get(theItem, 'value');
    value = _.isEmpty(value) ? theItem['default'] : value;
    return value === expanded.value !== expanded.negate;
  },

  filterGroups: function filterGroups(groups, filters) {
    _.remove(groups, function (group) {
      return !_.isObject(group);
    });
    return _(groups).map(function (group) {
      if (!ConfigService.evaluateFilters(_.get(group, 'filters'), filters)) {
        return null;
      }
      group.items = _(_.get(group, 'items', [])).map(function (item) {
        if (!ConfigService.evaluateFilters(_.get(item, 'filters'), filters)) {
          return null;
        }
        return item;
      }).without(null).value();
      return group;
    }).without(null).value();
  },

  evaluateFilters: function evaluateFilters(assertions, filters) {
    return !_.some(assertions, function (when) {
      var expanded = ConfigService.expandWhen(when);
      if (_.has(filters, expanded.key)) {
        var values = expanded.value.split(',');
        return !(expanded.negate !== _.some(values, function (value) {
          return filters[expanded.key] === value;
        }));
      }
      return false;
    });
  },

  expandWhen: function expandWhen(when) {
    var expanded = {
      key: '',
      value: '',
      negate: false
    };
    if (!when || typeof when !== 'string') {
      return expanded;
    }
    var parts = when.split('=');
    if (parts.length < 2) {
      expanded.value = when;
      return expanded;
    }
    expanded.key = parts.shift();
    expanded.value = parts.join('=');
    if (expanded.key.substr(expanded.key.length - 1) === '!') {
      expanded.key = expanded.key.substr(0, expanded.key.length - 1);
      expanded.negate = true;
    }
    return expanded;
  },

  isOldStyleWhen: function isOldStyleWhen(when) {
    if (!when || typeof when !== 'string') {
      return false;
    }
    var parts = when.split('=');
    if (parts.length < 2) {
      return false;
    }
    return true;
  },

  isVisible: function isVisible(groups, obj) {
    return !obj.hidden && ConfigService.isEnabled(groups, obj);
  },

  isEnabled: function isEnabled(groups, obj) {
    var when = _.get(obj, 'when');
    return when ? ConfigService.evaluateWhen(groups, when) : true;
  },

  parseBool: function parseBool(str) {
    switch (str) {
      case '1':
      case 't':
      case 'T':
      case 'true':
      case 'TRUE':
      case 'True':
        return true;
      case '0':
      case 'f':
      case 'F':
      case 'false':
      case 'FALSE':
      case 'False':
        return false;
      default:
        return false;
    }
  }
};

module.exports = ConfigService;
//# sourceMappingURL=ConfigService.js.map
